@@ -30,7 +30,7 @@
 #include <gmodule.h>
 #include "girepository.h"
 #include "gitypelib-internal.h"
-#include "ginfo.h"
+#include "girepository-private.h"
 #include "glib-compat.h"
 
 #include "config.h"
@@ -42,8 +42,8 @@ static GSList *override_search_path = NULL;
 
 struct _GIRepositoryPrivate
 {
-  GHashTable *typelibs; /* (string) namespace -> GTypelib */
-  GHashTable *lazy_typelibs; /* (string) namespace-version -> GTypelib */
+  GHashTable *typelibs; /* (string) namespace -> GITypelib */
+  GHashTable *lazy_typelibs; /* (string) namespace-version -> GITypelib */
   GHashTable *info_by_gtype; /* GType -> GIBaseInfo */
 };
 
@@ -131,6 +131,9 @@ init_globals (void)
           g_free (custom_dirs);
         }
 
+      if (override_search_path != NULL)
+        override_search_path = g_slist_reverse (override_search_path);
+
       libdir = GOBJECT_INTROSPECTION_LIBDIR;
 
       typelib_dir = g_build_filename (libdir, "girepository-1.0", NULL);
@@ -165,19 +168,21 @@ g_irepository_get_search_path (void)
   return search_path;
 }
 
-static
-GSList *
+static GSList *
 build_search_path_with_overrides (void)
 {
-	GSList *result;
-	if (override_search_path != NULL)
-	  {
-		result = g_slist_copy (override_search_path);
-		g_slist_last (result)->next = g_slist_copy (search_path);
-	  }
-	else
-	  result = g_slist_copy (search_path);
-	return result;
+  GSList *result;
+
+  init_globals ();
+
+  if (override_search_path != NULL)
+    {
+      result = g_slist_copy (override_search_path);
+      g_slist_last (result)->next = g_slist_copy (search_path);
+    }
+  else
+    result = g_slist_copy (search_path);
+  return result;
 }
 
 static char *
@@ -190,7 +195,7 @@ build_typelib_key (const char *name, const char *source)
 }
 
 static char **
-get_typelib_dependencies (GTypelib *typelib)
+get_typelib_dependencies (GITypelib *typelib)
 {
   Header *header;
   const char *dependencies_glob;
@@ -215,8 +220,8 @@ get_repository (GIRepository *repository)
     return default_repository;
 }
 
-static GTypelib *
-check_version_conflict (GTypelib *typelib,
+static GITypelib *
+check_version_conflict (GITypelib *typelib,
 			const gchar *namespace,
 			const gchar *expected_version,
 			char       **version_conflict)
@@ -246,7 +251,7 @@ check_version_conflict (GTypelib *typelib,
   return typelib;
 }
 
-static GTypelib *
+static GITypelib *
 get_registered_status (GIRepository *repository,
 		       const char   *namespace,
 		       const char   *version,
@@ -254,7 +259,7 @@ get_registered_status (GIRepository *repository,
 		       gboolean     *lazy_status,
 		       char        **version_conflict)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   repository = get_repository (repository);
   if (lazy_status)
     *lazy_status = FALSE;
@@ -271,7 +276,7 @@ get_registered_status (GIRepository *repository,
   return check_version_conflict (typelib, namespace, version, version_conflict);
 }
 
-static GTypelib *
+static GITypelib *
 get_registered (GIRepository *repository,
 		const char   *namespace,
 		const char   *version)
@@ -281,7 +286,7 @@ get_registered (GIRepository *repository,
 
 static gboolean
 load_dependencies_recurse (GIRepository *repository,
-			   GTypelib     *typelib,
+			   GITypelib     *typelib,
 			   GError      **error)
 {
   char **dependencies;
@@ -321,7 +326,7 @@ static const char *
 register_internal (GIRepository *repository,
 		   const char   *source,
 		   gboolean      lazy,
-		   GTypelib     *typelib,
+		   GITypelib     *typelib,
 		   GError      **error)
 {
   Header *header;
@@ -369,7 +374,7 @@ register_internal (GIRepository *repository,
 
 /**
  * g_irepository_get_dependencies:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace of interest
  *
  * Return an array of all (transitive) dependencies for namespace
@@ -379,13 +384,13 @@ register_internal (GIRepository *repository,
  * Note: The namespace must have already been loaded using a function
  * such as #g_irepository_require before calling this function.
  *
- * Returns: Zero-terminated string array of versioned dependencies
+ * Returns: (transfer full): Zero-terminated string array of versioned dependencies
  */
 char **
 g_irepository_get_dependencies (GIRepository *repository,
 				const char *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -399,7 +404,7 @@ g_irepository_get_dependencies (GIRepository *repository,
 
 const char *
 g_irepository_load_typelib (GIRepository *repository,
-			    GTypelib     *typelib,
+			    GITypelib     *typelib,
 			    GIRepositoryLoadFlags flags,
 			    GError      **error)
 {
@@ -435,7 +440,7 @@ g_irepository_load_typelib (GIRepository *repository,
 
 /**
  * g_irepository_is_registered:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace of interest
  * @version: (allow-none): Required version, may be %NULL for latest
  *
@@ -481,7 +486,7 @@ g_irepository_get_default (void)
 
 /**
  * g_irepository_get_n_infos:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the number of metadata entries in
@@ -494,7 +499,7 @@ gint
 g_irepository_get_n_infos (GIRepository *repository,
 			   const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   gint n_interfaces = 0;
 
   g_return_val_if_fail (namespace != NULL, -1);
@@ -526,7 +531,7 @@ find_interface (gpointer key,
 		gpointer data)
 {
   gint i;
-  GTypelib *typelib = (GTypelib *)value;
+  GITypelib *typelib = (GITypelib *)value;
   Header *header = (Header *) typelib->data;
   IfaceData *iface_data = (IfaceData *)data;
   gint index;
@@ -606,15 +611,15 @@ find_interface (gpointer key,
   if (index != 0)
     {
       entry = g_typelib_get_dir_entry (typelib, index);
-      iface_data->iface = g_info_new_full (entry->blob_type,
-					   iface_data->repo,
-					   NULL, typelib, entry->offset);
+      iface_data->iface = _g_info_new_full (entry->blob_type,
+				 	    iface_data->repo,
+					    NULL, typelib, entry->offset);
     }
 }
 
 /**
  * g_irepository_get_info:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  * @index: Offset into namespace metadata for entry
  *
@@ -622,7 +627,7 @@ find_interface (gpointer key,
  * given namespace @namespace_.  The namespace must have
  * already been loaded before calling this function.
  *
- * Returns: #GIBaseInfo containing metadata
+ * Returns: (transfer full): #GIBaseInfo containing metadata
  */
 GIBaseInfo *
 g_irepository_get_info (GIRepository *repository,
@@ -630,7 +635,7 @@ g_irepository_get_info (GIRepository *repository,
 			gint          index)
 {
   IfaceData data;
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -653,7 +658,7 @@ g_irepository_get_info (GIRepository *repository,
 
 /**
  * g_irepository_find_by_gtype:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @gtype: GType to search for
  *
  * Searches all loaded namespaces for a particular #GType.  Note that
@@ -663,7 +668,7 @@ g_irepository_get_info (GIRepository *repository,
  * arbitrary GType - thus, this function will operate most reliably
  * when you know the GType to originate from be from a loaded namespace.
  *
- * Returns: #GIBaseInfo representing metadata about @type, or %NULL
+ * Returns: (transfer full): #GIBaseInfo representing metadata about @type, or %NULL
  */
 GIBaseInfo *
 g_irepository_find_by_gtype (GIRepository *repository,
@@ -707,7 +712,7 @@ g_irepository_find_by_gtype (GIRepository *repository,
 
 /**
  * g_irepository_find_by_name:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace which will be searched
  * @name: Entry name to find
  *
@@ -716,7 +721,7 @@ g_irepository_find_by_gtype (GIRepository *repository,
  * #g_irepository_require once to load the namespace, or otherwise
  * ensure the namespace has already been loaded.
  *
- * Returns: #GIBaseInfo representing metadata about @name, or %NULL
+ * Returns: (transfer full): #GIBaseInfo representing metadata about @name, or %NULL
  */
 GIBaseInfo *
 g_irepository_find_by_name (GIRepository *repository,
@@ -724,7 +729,7 @@ g_irepository_find_by_name (GIRepository *repository,
 			    const gchar  *name)
 {
   IfaceData data;
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -756,8 +761,8 @@ collect_namespaces (gpointer key,
 }
 
 /**
- * g_irepository_get_namespaces:
- * @repository: A #GIRepository, may be %NULL for the default
+ * g_irepository_get_loaded_namespaces:
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  *
  * Return the list of currently loaded namespaces.
  *
@@ -786,7 +791,7 @@ g_irepository_get_loaded_namespaces (GIRepository *repository)
 
 /**
  * g_irepository_get_version:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the loaded version associated with the given
@@ -801,7 +806,7 @@ const gchar *
 g_irepository_get_version (GIRepository *repository,
 			   const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace != NULL, NULL);
@@ -818,7 +823,7 @@ g_irepository_get_version (GIRepository *repository,
 
 /**
  * g_irepository_get_shared_library:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the full path to the shared C library
@@ -835,7 +840,7 @@ const gchar *
 g_irepository_get_shared_library (GIRepository *repository,
 				  const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace != NULL, NULL);
@@ -855,7 +860,7 @@ g_irepository_get_shared_library (GIRepository *repository,
 
 /**
  * g_irepository_get_c_prefix
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the "C prefix", or the C level namespace
@@ -871,7 +876,7 @@ const gchar *
 g_irepository_get_c_prefix (GIRepository *repository,
                             const gchar  *namespace_)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace_ != NULL, NULL);
@@ -891,7 +896,7 @@ g_irepository_get_c_prefix (GIRepository *repository,
 
 /**
  * g_irepository_get_typelib_path
- * @repository: Repository, may be %NULL for the default
+ * @repository: (allow-none): Repository, may be %NULL for the default
  * @namespace_: GI namespace to use, e.g. "Gtk"
  *
  * If namespace @namespace_ is loaded, return the full path to the
@@ -926,9 +931,9 @@ g_irepository_get_typelib_path (GIRepository *repository,
 static GMappedFile *
 find_namespace_version (const gchar  *namespace,
 			const gchar  *version,
+			GSList       *search_path,
 			gchar       **path_ret)
 {
-  GSList *tmp_path;
   GSList *ldir;
   GError *error = NULL;
   GMappedFile *mfile = NULL;
@@ -936,8 +941,7 @@ find_namespace_version (const gchar  *namespace,
 
   fname = g_strdup_printf ("%s-%s.typelib", namespace, version);
 
-  tmp_path = build_search_path_with_overrides ();
-  for (ldir = tmp_path; ldir; ldir = ldir->next)
+  for (ldir = search_path; ldir; ldir = ldir->next)
     {
       char *path = g_build_filename (ldir->data, fname, NULL);
 
@@ -952,7 +956,6 @@ find_namespace_version (const gchar  *namespace,
       break;
     }
   g_free (fname);
-  g_slist_free (tmp_path);
   return mfile;
 }
 
@@ -1046,29 +1049,23 @@ free_candidate (struct NamespaceVersionCandidadate *candidate)
   g_slice_free (struct NamespaceVersionCandidadate, candidate);
 }
 
-static GMappedFile *
-find_namespace_latest (const gchar  *namespace,
-		       gchar       **version_ret,
-		       gchar       **path_ret)
+static GSList *
+enumerate_namespace_versions (const gchar *namespace,
+			      GSList      *search_path)
 {
-  GSList *tmp_path;
-  GSList *ldir;
-  GError *error = NULL;
+  GSList *candidates = NULL;
+  GHashTable *found_versions = g_hash_table_new (g_str_hash, g_str_equal);
   char *namespace_dash;
   char *namespace_typelib;
-  GSList *candidates = NULL;
-  GMappedFile *result = NULL;
+  GSList *ldir;
+  GError *error = NULL;
   int index;
 
-  *version_ret = NULL;
-  *path_ret = NULL;
-
   namespace_dash = g_strdup_printf ("%s-", namespace);
   namespace_typelib = g_strdup_printf ("%s.typelib", namespace);
 
   index = 0;
-  tmp_path = build_search_path_with_overrides ();
-  for (ldir = tmp_path; ldir; ldir = ldir->next)
+  for (ldir = search_path; ldir; ldir = ldir->next)
     {
       GDir *dir;
       const char *dirname;
@@ -1102,6 +1099,10 @@ find_namespace_latest (const gchar  *namespace,
 	  else
 	    continue;
 
+	  if (g_hash_table_lookup (found_versions, version) != NULL)
+	    continue;
+	  g_hash_table_insert (found_versions, version, version);
+
 	  path = g_build_filename (dirname, entry, NULL);
 	  mfile = g_mapped_file_new (path, FALSE, &error);
 	  if (mfile == NULL)
@@ -1122,6 +1123,27 @@ find_namespace_latest (const gchar  *namespace,
       index++;
     }
 
+  g_free (namespace_dash);
+  g_free (namespace_typelib);
+  g_hash_table_destroy (found_versions);
+
+  return candidates;
+}
+
+static GMappedFile *
+find_namespace_latest (const gchar  *namespace,
+		       GSList       *search_path,
+		       gchar       **version_ret,
+		       gchar       **path_ret)
+{
+  GSList *candidates;
+  GMappedFile *result = NULL;
+
+  *version_ret = NULL;
+  *path_ret = NULL;
+
+  candidates = enumerate_namespace_versions (namespace, search_path);
+
   if (candidates != NULL)
     {
       struct NamespaceVersionCandidadate *elected;
@@ -1138,39 +1160,66 @@ find_namespace_latest (const gchar  *namespace,
       g_slist_foreach (candidates, (GFunc) free_candidate, NULL);
       g_slist_free (candidates);
     }
-  g_free (namespace_dash);
-  g_free (namespace_typelib);
-  g_slist_free (tmp_path);
   return result;
 }
 
 /**
- * g_irepository_require:
- * @repository: (allow-none): Repository, may be %NULL for the default
- * @namespace_: GI namespace to use, e.g. "Gtk"
- * @version: (allow-none): Version of namespace, may be %NULL for latest
- * @flags: Set of %GIRepositoryLoadFlags, may be 0
- * @error: a #GError.
+ * g_irepository_enumerate_versions:
+ * @repository: (allow-none): the repository
+ * @namespace_: GI namespace, e.g. "Gtk"
  *
- * Force the namespace @namespace_ to be loaded if it isn't already.
- * If @namespace_ is not loaded, this function will search for a
- * ".typelib" file using the repository search path.  In addition, a
- * version @version of namespace may be specified.  If @version is
- * not specified, the latest will be used.
+ * Obtain an unordered list of versions (either currently loaded or
+ * available) for @namespace_ in this @repository.
  *
- * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ * Returns: (element-type utf8) (transfer full): the array of versions.
  */
-GTypelib *
-g_irepository_require (GIRepository  *repository,
-		       const gchar   *namespace,
-		       const gchar   *version,
-		       GIRepositoryLoadFlags flags,
-		       GError       **error)
+GList *
+g_irepository_enumerate_versions (GIRepository *repository,
+			 const gchar  *namespace_)
+{
+  GList *ret = NULL;
+  GSList *search_path;
+  GSList *candidates, *link;
+  const gchar *loaded_version;
+
+  search_path = build_search_path_with_overrides ();
+  candidates = enumerate_namespace_versions (namespace_, search_path);
+  g_slist_free (search_path);
+
+  for (link = candidates; link; link = link->next)
+    {
+      struct NamespaceVersionCandidadate *candidate = link->data;
+      ret = g_list_prepend (ret, g_strdup (candidate->version));
+      free_candidate (candidate);
+    }
+  g_slist_free (candidates);
+
+  /* The currently loaded version of a namespace is also part of the
+   * available versions, as it could have been loaded using
+   * require_private().
+   */
+  if (g_irepository_is_registered (repository, namespace_, NULL))
+    {
+      loaded_version = g_irepository_get_version (repository, namespace_);
+      if (loaded_version && !g_list_find_custom (ret, loaded_version, g_str_equal))
+        ret = g_list_prepend (ret, g_strdup (loaded_version));
+    }
+
+  return ret;
+}
+
+static GITypelib *
+require_internal (GIRepository  *repository,
+		  const gchar   *namespace,
+		  const gchar   *version,
+		  GIRepositoryLoadFlags flags,
+		  GSList        *search_path,
+		  GError       **error)
 {
   GMappedFile *mfile;
-  GTypelib *ret = NULL;
+  GITypelib *ret = NULL;
   Header *header;
-  GTypelib *typelib = NULL;
+  GITypelib *typelib = NULL;
   const gchar *typelib_namespace, *typelib_version;
   gboolean allow_lazy = (flags & G_IREPOSITORY_LOAD_FLAG_LAZY) > 0;
   gboolean is_lazy;
@@ -1198,12 +1247,14 @@ g_irepository_require (GIRepository  *repository,
 
   if (version != NULL)
     {
-      mfile = find_namespace_version (namespace, version, &path);
+      mfile = find_namespace_version (namespace, version,
+				      search_path, &path);
       tmp_version = g_strdup (version);
     }
   else
     {
-      mfile = find_namespace_latest (namespace, &tmp_version, &path);
+      mfile = find_namespace_latest (namespace, search_path,
+				     &tmp_version, &path);
     }
 
   if (mfile == NULL)
@@ -1221,7 +1272,19 @@ g_irepository_require (GIRepository  *repository,
       goto out;
     }
 
-  typelib = g_typelib_new_from_mapped_file (mfile);
+  {
+    GError *temp_error = NULL;
+    typelib = g_typelib_new_from_mapped_file (mfile, &temp_error);
+    if (!typelib)
+      {
+	g_set_error (error, G_IREPOSITORY_ERROR,
+		     G_IREPOSITORY_ERROR_TYPELIB_NOT_FOUND,
+		     "Failed to load typelib file '%s' for namespace '%s': %s",
+		     path, namespace, temp_error->message);
+	g_clear_error (&temp_error);
+	goto out;
+      }
+  }
   header = (Header *) typelib->data;
   typelib_namespace = g_typelib_get_string (typelib, header->namespace);
   typelib_version = g_typelib_get_string (typelib, header->nsversion);
@@ -1233,6 +1296,7 @@ g_irepository_require (GIRepository  *repository,
 		   "Typelib file %s for namespace '%s' contains "
 		   "namespace '%s' which doesn't match the file name",
 		   path, namespace, typelib_namespace);
+      g_typelib_free (typelib);
       goto out;
     }
   if (version != NULL && strcmp (typelib_version, version) != 0)
@@ -1242,6 +1306,7 @@ g_irepository_require (GIRepository  *repository,
 		   "Typelib file %s for namespace '%s' contains "
 		   "version '%s' which doesn't match the expected version '%s'",
 		   path, namespace, typelib_version, version);
+      g_typelib_free (typelib);
       goto out;
     }
 
@@ -1258,14 +1323,86 @@ g_irepository_require (GIRepository  *repository,
   return ret;
 }
 
+/**
+ * g_irepository_require:
+ * @repository: (allow-none): Repository, may be %NULL for the default
+ * @namespace_: GI namespace to use, e.g. "Gtk"
+ * @version: (allow-none): Version of namespace, may be %NULL for latest
+ * @flags: Set of %GIRepositoryLoadFlags, may be 0
+ * @error: a #GError.
+ *
+ * Force the namespace @namespace_ to be loaded if it isn't already.
+ * If @namespace_ is not loaded, this function will search for a
+ * ".typelib" file using the repository search path.  In addition, a
+ * version @version of namespace may be specified.  If @version is
+ * not specified, the latest will be used.
+ *
+ * Returns: (transfer none): a pointer to the #GITypelib if successful, %NULL otherwise
+ */
+GITypelib *
+g_irepository_require (GIRepository  *repository,
+		       const gchar   *namespace,
+		       const gchar   *version,
+		       GIRepositoryLoadFlags flags,
+		       GError       **error)
+{
+  GSList *search_path;
+  GITypelib *typelib;
+
+  search_path = build_search_path_with_overrides ();
+  typelib = require_internal (repository, namespace, version, flags,
+			      search_path, error);
+  g_slist_free (search_path);
+
+  return typelib;
+}
+
+/**
+ * g_irepository_require_private:
+ * @repository: (allow-none): Repository, may be %NULL for the default
+ * @typelib_dir: Private directory where to find the requested typelib
+ * @namespace_: GI namespace to use, e.g. "Gtk"
+ * @version: (allow-none): Version of namespace, may be %NULL for latest
+ * @flags: Set of %GIRepositoryLoadFlags, may be 0
+ * @error: a #GError.
+ *
+ * Force the namespace @namespace_ to be loaded if it isn't already.
+ * If @namespace_ is not loaded, this function will search for a
+ * ".typelib" file within the private directory only. In addition, a
+ * version @version of namespace should be specified.  If @version is
+ * not specified, the latest will be used.
+ *
+ * Returns: (transfer none): a pointer to the #GITypelib if successful, %NULL otherwise
+ */
+GITypelib *
+g_irepository_require_private (GIRepository  *repository,
+			       const gchar   *typelib_dir,
+			       const gchar   *namespace,
+			       const gchar   *version,
+			       GIRepositoryLoadFlags flags,
+			       GError       **error)
+{
+  GSList search_path = { (gpointer) typelib_dir, NULL };
+
+  return require_internal (repository, namespace, version, flags,
+			   &search_path, error);
+}
+
 static gboolean
 g_irepository_introspect_cb (const char *option_name,
 			     const char *value,
 			     gpointer data,
 			     GError **error)
 {
-  gboolean ret = g_irepository_dump (value, error);
-  exit (ret ? 0 : 1);
+  GError *tmp_error = NULL;
+  gboolean ret = g_irepository_dump (value, &tmp_error);
+  if (!ret)
+    {
+      g_error ("Failed to extract GType data: %s",
+	       tmp_error->message);
+      exit (1);
+    }
+  exit (0);
 }
 
 static const GOptionEntry introspection_args[] = {
@@ -1275,6 +1412,15 @@ static const GOptionEntry introspection_args[] = {
   { NULL }
 };
 
+/**
+ * g_irepository_get_option_group: (skip)
+ *
+ * Obtain the option group for girepository, it's used
+ * by the dumper and for programs that wants to provide
+ * introspection information
+ *
+ * Returns: (transfer full): the option group
+ */
 GOptionGroup *
 g_irepository_get_option_group (void)
 {
@@ -1294,6 +1440,14 @@ g_irepository_error_quark (void)
   return quark;
 }
 
+/**
+ * g_type_tag_to_string:
+ * @type: the type_tag
+ *
+ * Obtain a string representation of @type
+ *
+ * Returns: the string
+ */
 const gchar*
 g_type_tag_to_string (GITypeTag type)
 {
@@ -1302,45 +1456,27 @@ g_type_tag_to_string (GITypeTag type)
     case GI_TYPE_TAG_VOID:
       return "void";
     case GI_TYPE_TAG_BOOLEAN:
-      return "boolean";
+      return "gboolean";
     case GI_TYPE_TAG_INT8:
-      return "int8";
+      return "gint8";
     case GI_TYPE_TAG_UINT8:
-      return "uint8";
+      return "guint8";
     case GI_TYPE_TAG_INT16:
-      return "int16";
+      return "gint16";
     case GI_TYPE_TAG_UINT16:
-      return "uint16";
+      return "guint16";
     case GI_TYPE_TAG_INT32:
-      return "int32";
+      return "gint32";
     case GI_TYPE_TAG_UINT32:
-      return "uint32";
+      return "guint32";
     case GI_TYPE_TAG_INT64:
-      return "int64";
+      return "gint64";
     case GI_TYPE_TAG_UINT64:
-      return "uint64";
-    case GI_TYPE_TAG_SHORT:
-      return "short";
-    case GI_TYPE_TAG_USHORT:
-      return "ushort";
-    case GI_TYPE_TAG_INT:
-      return "int";
-    case GI_TYPE_TAG_UINT:
-      return "uint";
-    case GI_TYPE_TAG_LONG:
-      return "long";
-    case GI_TYPE_TAG_ULONG:
-      return "ulong";
-    case GI_TYPE_TAG_SSIZE:
-      return "ssize";
-    case GI_TYPE_TAG_SIZE:
-      return "size";
+      return "guint64";
     case GI_TYPE_TAG_FLOAT:
-      return "float";
+      return "gfloat";
     case GI_TYPE_TAG_DOUBLE:
-      return "double";
-    case GI_TYPE_TAG_TIME_T:
-      return "time_t";
+      return "gdouble";
     case GI_TYPE_TAG_GTYPE:
       return "GType";
     case GI_TYPE_TAG_UTF8:
@@ -1363,3 +1499,61 @@ g_type_tag_to_string (GITypeTag type)
       return "unknown";
     }
 }
+
+/**
+ * g_info_type_to_string:
+ * @type: the info type
+ *
+ * Obtain a string representation of @type
+ *
+ * Returns: the string
+ */
+const gchar*
+g_info_type_to_string (GIInfoType type)
+{
+  switch (type)
+    {
+    case GI_INFO_TYPE_INVALID:
+      return "invalid";
+    case GI_INFO_TYPE_FUNCTION:
+      return "function";
+    case GI_INFO_TYPE_CALLBACK:
+      return "callback";
+    case GI_INFO_TYPE_STRUCT:
+      return "struct";
+    case GI_INFO_TYPE_BOXED:
+      return "boxed";
+    case GI_INFO_TYPE_ENUM:
+      return "enum";
+    case GI_INFO_TYPE_FLAGS:
+      return "flags";
+    case GI_INFO_TYPE_OBJECT:
+      return "object";
+    case GI_INFO_TYPE_INTERFACE:
+      return "interface";
+    case GI_INFO_TYPE_CONSTANT:
+      return "constant";
+    case GI_INFO_TYPE_ERROR_DOMAIN:
+      return "error domain";
+    case GI_INFO_TYPE_UNION:
+      return "union";
+    case GI_INFO_TYPE_VALUE:
+      return "value";
+    case GI_INFO_TYPE_SIGNAL:
+      return "signal";
+    case GI_INFO_TYPE_VFUNC:
+      return "vfunc";
+    case GI_INFO_TYPE_PROPERTY:
+      return "property";
+    case GI_INFO_TYPE_FIELD:
+      return "field";
+    case GI_INFO_TYPE_ARG:
+      return "arg";
+    case GI_INFO_TYPE_TYPE:
+      return "type";
+    case GI_INFO_TYPE_UNRESOLVED:
+      return "unresolved";
+    default:
+      return "unknown";
+  }
+}
