@@ -78,7 +78,7 @@ g_ir_module_free (GIrModule *module)
 
 /**
  * g_ir_module_fatal:
- * @module: Current module
+ * @build: Current build
  * @line: Origin line number, or 0 if unknown
  * @msg: printf-format string
  * @args: Remaining arguments
@@ -86,12 +86,14 @@ g_ir_module_free (GIrModule *module)
  * Report a fatal error, then exit.
  */
 void
-g_ir_module_fatal (GIrModule  *module,
+g_ir_module_fatal (GIrTypelibBuild  *build,
                    guint       line,
                    const char *msg,
                    ...)
 {
+  GString *context;
   char *formatted;
+  GList *link;
 
   va_list args;
 
@@ -99,10 +101,27 @@ g_ir_module_fatal (GIrModule  *module,
 
   formatted = g_strdup_vprintf (msg, args);
 
-  if (line)
-    g_printerr ("%s-%s.gir:%d: error: %s\n", module->name, module->version, line, formatted);
-  else
-    g_printerr ("%s-%s.gir: error: %s\n", module->name, module->version, formatted);
+  context = g_string_new ("");
+  if (line > 0)
+    g_string_append_printf (context, "%d: ", line);
+  if (build->stack)
+    g_string_append (context, "In ");
+  for (link = g_list_last (build->stack); link; link = link->prev)
+    {
+      GIrNode *node = link->data;
+      const char *name = node->name;
+      if (name)
+	g_string_append (context, name);
+      if (link->prev)
+	g_string_append (context, ".");
+    }
+  if (build->stack)
+    g_string_append (context, ": ");
+
+  g_printerr ("%s-%s.gir:%serror: %s\n", build->module->name, 
+	      build->module->version,
+	      context->str, formatted);
+  g_string_free (context, TRUE);
 
   exit (1);
 
@@ -192,11 +211,21 @@ write_attributes (GIrModule *module,
   return wdata.count;
 }
 
-GTypelib *
-g_ir_module_build_typelib (GIrModule  *module,
-			     GList       *modules)
+static gint
+node_cmp_offset_func (gconstpointer a,
+                      gconstpointer b)
 {
-  GTypelib *typelib;
+  const GIrNode *na = a;
+  const GIrNode *nb = b;
+  return na->offset - nb->offset;
+}
+
+
+GITypelib *
+g_ir_module_build_typelib (GIrModule  *module)
+{
+  GError *error = NULL;
+  GITypelib *typelib;
   gsize length;
   guint i;
   GList *e;
@@ -209,7 +238,7 @@ g_ir_module_build_typelib (GIrModule  *module,
   guint32 size, offset, offset2, old_offset;
   GHashTable *strings;
   GHashTable *types;
-  GList *offset_ordered_nodes;
+  GList *nodes_with_attributes;
   char *dependencies;
   guchar *data;
 
@@ -242,7 +271,7 @@ g_ir_module_build_typelib (GIrModule  *module,
   _g_irnode_init_stats ();
   strings = g_hash_table_new (g_str_hash, g_str_equal);
   types = g_hash_table_new (g_str_hash, g_str_equal);
-  offset_ordered_nodes = NULL;
+  nodes_with_attributes = NULL;
   n_entries = g_list_length (module->entries);
 
   g_message ("%d entries (%d local), %d dependencies\n", n_entries, n_local_entries,
@@ -258,7 +287,6 @@ g_ir_module_build_typelib (GIrModule  *module,
       GIrNode *node = e->data;
 
       size += g_ir_node_get_full_size (node);
-      size += g_ir_node_get_attribute_size (node);
 
       /* Also reset the offset here */
       node->offset = 0;
@@ -281,7 +309,7 @@ g_ir_module_build_typelib (GIrModule  *module,
   /* fill in header */
   header = (Header *)data;
   memcpy (header, G_IR_MAGIC, 16);
-  header->major_version = 2;
+  header->major_version = 4;
   header->minor_version = 0;
   header->reserved = 0;
   header->n_entries = n_entries;
@@ -350,10 +378,10 @@ g_ir_module_build_typelib (GIrModule  *module,
 	  g_hash_table_destroy (types);
 
 	  /* Reset the cached offsets */
-	  for (link = offset_ordered_nodes; link; link = link->next)
+	  for (link = nodes_with_attributes; link; link = link->next)
 	    ((GIrNode *) link->data)->offset = 0;
 
-	  g_list_free (offset_ordered_nodes);
+	  g_list_free (nodes_with_attributes);
 	  strings = NULL;
 
 	  g_free (data);
@@ -383,16 +411,16 @@ g_ir_module_build_typelib (GIrModule  *module,
 	  entry->offset = offset;
 	  entry->name = write_string (node->name, strings, data, &offset2);
 
+	  memset (&build, 0, sizeof (build));
 	  build.module = module;
-	  build.modules = modules;
 	  build.strings = strings;
 	  build.types = types;
-	  build.offset_ordered_nodes = offset_ordered_nodes;
+	  build.nodes_with_attributes = nodes_with_attributes;
 	  build.n_attributes = header->n_attributes;
 	  build.data = data;
 	  g_ir_node_build_typelib (node, NULL, &build, &offset, &offset2);
 
-	  offset_ordered_nodes = build.offset_ordered_nodes;
+	  nodes_with_attributes = build.nodes_with_attributes;
 	  header->n_attributes = build.n_attributes;
 
 	  if (offset2 > old_offset + g_ir_node_get_full_size (node))
@@ -402,7 +430,8 @@ g_ir_module_build_typelib (GIrModule  *module,
       entry++;
     }
 
-  offset_ordered_nodes = g_list_reverse (offset_ordered_nodes);
+  /* GIBaseInfo expects the AttributeBlob array to be sorted on the field (offset) */
+  nodes_with_attributes = g_list_sort (nodes_with_attributes, node_cmp_offset_func);
 
   g_message ("header: %d entries, %d attributes", header->n_entries, header->n_attributes);
 
@@ -413,10 +442,9 @@ g_ir_module_build_typelib (GIrModule  *module,
   header->attributes = offset;
   offset2 = offset + header->n_attributes * header->attribute_blob_size;
 
-  for (e = offset_ordered_nodes; e; e = e->next)
+  for (e = nodes_with_attributes; e; e = e->next)
     {
       GIrNode *node = e->data;
-
       write_attributes (module, node, strings, data, &offset, &offset2);
     }
 
@@ -425,11 +453,16 @@ g_ir_module_build_typelib (GIrModule  *module,
   data = g_realloc (data, offset2);
   header = (Header*) data;
   length = header->size = offset2;
-  typelib = g_typelib_new_from_memory (data, length);
+  typelib = g_typelib_new_from_memory (data, length, &error);
+  if (!typelib)
+    {
+      g_error ("error building typelib: %s",
+	       error->message);
+    }
 
   g_hash_table_destroy (strings);
   g_hash_table_destroy (types);
-  g_list_free (offset_ordered_nodes);
+  g_list_free (nodes_with_attributes);
 
   return typelib;
 }
