@@ -84,6 +84,7 @@ class GDumpParser(object):
         for node in self._namespace.itervalues():
             if isinstance(node, ast.Function):
                 self._initparse_function(node)
+
         if self._namespace.name == 'GObject':
             for node in self._namespace.itervalues():
                 if isinstance(node, ast.Record):
@@ -131,6 +132,13 @@ class GDumpParser(object):
         for node in to_remove:
             self._namespace.remove(node)
 
+        if self._namespace.name == 'GLib':
+            variant = self._namespace.get('Variant')
+            assert variant is not None
+            variant.add_gtype('GVariant', 'g_variant_get_gtype')
+            # Work around scanner bug
+            variant.disguised = False
+
     # Helper functions
 
     def _execute_binary_get_tree(self):
@@ -220,13 +228,10 @@ blob containing data gleaned from GObject's primitive introspection."""
 
     def _initparse_gobject_record(self, record):
         # Special handling for when we're parsing GObject
-        internal_names = ("Object", 'InitiallyUnowned')
-        if record.name in internal_names:
+        if record.name in ("Object", 'InitiallyUnowned'):
             self._create_gobject(record)
-            return
-        if record.name == 'InitiallyUnownedClass':
+        elif record.name == 'InitiallyUnownedClass':
             record.fields = self._namespace.get('ObjectClass').fields
-        self._namespace.append(record, replace=True)
 
     # Introspection over the data we get from the dynamic
     # GObject/GType system out of the binary
@@ -279,6 +284,12 @@ blob containing data gleaned from GObject's primitive introspection."""
         get_type = xmlnode.attrib['get-type']
         (ns, name) = self._transformer.split_csymbol(get_type)
         assert ns is self._namespace
+        if name == 'get_type':
+            message.fatal("""The GObject name %r isn't compatibile
+with the configured identifier prefixes:
+  %r
+The class would have no name.  Most likely you want to specify a
+different --identifier-prefix.""" % (xmlnode.attrib['name'], self._namespace.identifier_prefixes))
         assert name.endswith('_get_type')
         return (get_type, name[:-len('_get_type')])
 
@@ -339,8 +350,23 @@ blob containing data gleaned from GObject's primitive introspection."""
         else:
             self._namespace.append(node, replace=True)
 
+    ## WORKAROUND ##
+    # https://bugzilla.gnome.org/show_bug.cgi?id=550616
+    def _introspect_boxed_gstreamer_workaround(self, xmlnode):
+        node = ast.Boxed('ParamSpecMiniObject', gtype_name='GParamSpecMiniObject',
+                         get_type='gst_param_spec_mini_object_get_type',
+                         c_symbol_prefix='param_spec_mini_object')
+        self._boxed_types[node.gtype_name] = node
+
     def _introspect_boxed(self, xmlnode):
         type_name = xmlnode.attrib['name']
+
+        # Work around GStreamer legacy naming issue
+        # https://bugzilla.gnome.org/show_bug.cgi?id=550616
+        if type_name == 'GParamSpecMiniObject':
+            self._introspect_boxed_gstreamer_workaround(xmlnode)
+            return
+
         try:
             name = self._transformer.strip_identifier(type_name)
         except TransformerException, e:
